import * as _nuxt_schema from '@nuxt/schema';
import { M as ModuleContext, a as ModuleOptions } from './shared/nuxt-graphql-middleware.-BeiPV4H.mjs';
import { OperationResponseError } from '../dist/runtime/types.js';
import { HookResult } from 'nuxt/schema';
import { GeneratorOutput } from 'graphql-typescript-deluxe';
import 'graphql';
import '@nuxt/kit';
import 'h3';
import '@graphql-codegen/plugin-helpers';
import '@graphql-codegen/schema-ast';

type BuildHookContext = {
    output: GeneratorOutput;
};

declare const _default: _nuxt_schema.NuxtModule<ModuleOptions, ModuleOptions, false>;

declare module '#app' {
    interface RuntimeNuxtHooks {
        /**
         * Emitted when any GraphQL response contains errors.
         */
        'nuxt-graphql-middleware:errors': (errors: OperationResponseError) => HookResult;
    }
}
declare module 'vite/types/customEvent.d.ts' {
    interface CustomEventMap {
        /**
         * Emitted when GraphQL operations have been updated.
         */
        'nuxt-graphql-middleware:reload': {
            operations: string[];
        };
    }
}
declare module '@nuxt/schema' {
    interface AppConfig {
        graphqlMiddleware: {
            /**
             * Whether the client cache is enabled.
             */
            clientCacheEnabled: boolean;
            /**
             * The max number of items in the cache.
             */
            clientCacheMaxSize: number;
        };
    }
    interface Nuxt {
        /**
         * The nuxt-graphql-middleware module context.
         */
        _nuxt_graphql_middleware?: ModuleContext;
    }
    interface NuxtHooks {
        /**
         * Called once right before the documents are initialised.
         *
         * Use this hook to add any additional documents based on for example the parsed schema.
         *
         * @example
         *
         * ```typescript`
         * export default defineNuxtConfig({
         *   hooks: {
         *     'nuxt-graphql-middleware:init': (ctx) => {
         *       if (ctx.schemaHasType('Comment')) {
         *         ctx.addDocument(
         *           'queryFromHook',
         *           `query loadComments { author subject date body }`
         *         )
         *       }
         *     },
         *   },
         * })
         * ```
         */
        'nuxt-graphql-middleware:init': (ctx: ModuleContext) => void | Promise<void>;
        /**
         * Called when building the state and templates.
         *
         * Generally this hook is called from within the builder:watch event, when
         * a GraphQL file change was detected.
         *
         * The received argument is the output from the graphql-typescript-deluxe
         * generator.
         */
        'nuxt-graphql-middleware:build': (ctx: BuildHookContext) => void | Promise<void>;
    }
}

export { ModuleOptions, _default as default };
