import type { FetchOptions } from 'ofetch';
import type { RequestCacheOptions } from './../types.js';
import type { GraphqlClientContext } from '#nuxt-graphql-middleware/client-options';
import type { Query, Mutation } from '#nuxt-graphql-middleware/operation-types';
import type { NuxtApp } from '#app';
import type { AppConfig } from 'nuxt/schema';
import { GraphqlMiddlewareCache } from './ClientCache.js';
export type GraphqlComposableOptions = {
    fetchOptions?: FetchOptions;
    graphqlCaching?: RequestCacheOptions;
    clientContext?: Partial<GraphqlClientContext>;
};
export type GetQueryArgs<K extends keyof Query, Q extends Query[K] = Query[K]> = Q['variables'] extends null ? [K, (null | undefined)?, GraphqlComposableOptions?] : Q['needsVariables'] extends true ? [K, Q['variables'], GraphqlComposableOptions?] : [K, (Q['variables'] | null)?, GraphqlComposableOptions?];
export type GetMutationArgs<K extends keyof Mutation, M extends Mutation[K] = Mutation[K]> = M['needsVariables'] extends true ? [K, M['variables'], GraphqlComposableOptions?] : [K, (M['variables'] | null)?, GraphqlComposableOptions?];
export type GetQueryResult<K extends keyof Query, Q extends Query[K] = Query[K]> = Q['response'];
export type GetMutationResult<K extends keyof Mutation, M extends Mutation[K] = Mutation[K]> = M['response'];
export type QueryObjectArgs<K extends keyof Query, Q extends Query[K] = Query[K]> = Q['needsVariables'] extends true ? {
    name: K;
    fetchOptions?: FetchOptions;
    graphqlCaching?: RequestCacheOptions;
    clientContext?: Partial<GraphqlClientContext>;
    variables: Q['variables'];
} : {
    name: K;
    variables?: Q['variables'] | null;
    fetchOptions?: FetchOptions;
    graphqlCaching?: RequestCacheOptions;
    clientContext?: Partial<GraphqlClientContext>;
};
export type MutationObjectArgs<K extends keyof Mutation, M extends Mutation[K] = Mutation[K]> = M['needsVariables'] extends true ? {
    name: K;
    variables: M['variables'];
    fetchOptions?: FetchOptions;
    clientContext?: Partial<GraphqlClientContext>;
} : {
    name: K;
    variables?: M['variables'] | null;
    fetchOptions?: FetchOptions;
    clientContext?: Partial<GraphqlClientContext>;
};
export type PickFrom<T, K extends Array<string>> = T extends Array<any> ? T : T extends Record<string, any> ? keyof T extends K[number] ? T : K[number] extends never ? T : Pick<T, K[number]> : T;
export type KeysOf<T> = Array<T extends T ? (keyof T extends string ? keyof T : never) : never>;
export declare function encodeContext(context: Record<string, string | null | undefined>): Record<string, string>;
/**
 * Sort an object defining query params alphabetically.
 */
export declare function sortQueryParams(obj: Record<string, string>): Record<string, string>;
export declare function getOrCreateClientCache(app: NuxtApp, config: AppConfig): GraphqlMiddlewareCache | undefined;
